require 'concurrent'

module ServicePacker
  class ServicePackerStore
    @stores = Concurrent::Map.new

    class << self
      def stores
        @stores
      end

      def register(klass, override = false)
        raise "Cannot register #{klass}" unless klass.is_a?(Class)

        case override
          when FalseClass then stores[klass.to_s] ||= ServicePacker::Service.new
          when TrueClass then stores[klass.to_s] = ServicePacker::Service.new
          else raise "Don't know how to do with #{override}"
        end
      end
      alias_method :[]=, :register

      def fetch(klass)
        stores[klass.to_s]
      end
      alias_method :[], :fetch

      def unregister(klass)
        stores.delete(klass.to_s)
      end
    end
  end
end
